;; Generated package description from codex-ide.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "codex-ide" "0.1.1.0.20260813.0" "Run Codex CLI in a terminal" '((emacs "29.1") (compat "29.1.4.2") (keymap-popup "0.4.0") (eat "0.9.4")) :commit "d11a143ac52894eb25bbb43d877c8761f2f01244" :maintainer '("Thanos Apollo" . "public@thanosapollo.org") :keywords '("ai" "codex" "tools" "terminal") :url "https://git.thanosapollo.org/emacs-codex-ide")
